%
% Calculo del tiempo medio entre falsas alarmas, ARL0, del algoritmo
% Shewart Control Chart para detectar cambios bruscos en la media de seales 
% i.i.d. Gaussianas. Todos los parametros son conocidos salvo el instante del cambio.
%
% Hace simulacion de Montecarlo promediando el tiempo en que se produce la primera
% falsa alarma para muchas secuencias sin cambio en la media (bajo H0). 
%
% Cada secuencia se compone de un numero variable de bloques de N muestras y se toma
% la decision al final de cada bloque.

close all

% Datos:
mu0 = 0; % media antes del cambio
mu1 = 1; % media despues del cambio
vari = 1; % varianza

% Hiperparametros del algoritmo Shewart Control Chart
umbral = 0.1; % umbral de deteccion
N=16; % Longitud de los bloques

% Simulacion de Montecarlo
Ns=1e5; % Numero de secuencias analizadas

% Inicializa el vector de instantes de deteccion de falsas alarmas
Md = zeros(Ns,1);

C1=(mu0^2-mu1^2)/(2*vari);
C2=(mu1-mu0)/vari;

for ns=1:Ns
   
    % Primer bloque
    x = mu0+sqrt(vari)*randn(N,1);
    S=C1+C2*x;
    T=sum(S)/N;
    if T > umbral % deteccion de salto en el primer bloque
        Md(ns) = N;
    end
    
    % Siguientes bloques
    k=2;
    while Md(ns) ==0
        x = mu0+sqrt(vari)*randn(N,1);
        S=C1+C2*x;
        T=sum(S)/N;
        if T > umbral
            Md(ns) = k*N;
        end
        k=k+1;
    end
    
    %disp(['realizacion: ',num2str(ns),'      Md=',num2str(Md(ns))])
    
end

ARL0=mean(Md);

disp(' ')
disp(['Longitud bloques: ',num2str(N),'      umbral=',num2str(umbral)])
disp(['ARL0 =',num2str(ARL0)])
