%
% Calculo del retardo medio de deteccion del cambio , ARL1, del algoritmo
% Shewart Control Chart para detectar cambios bruscos en la media de seales 
% i.i.d. Gaussianas. Todos los parametros son conocidos salvo el instante del cambio.
%
% Hace simulacion de Montecarlo promediando el retardo de deteccion
% para muchas secuencias. 
%
% El instante de cambio se produce en una muestra aleatoria del primer bloque. 
%
% Cada secuencia se compone de un numero variable de bloques de N muestras y se toma
% la decision al final de cada bloque.

close all

% Datos:
mu0 = 0; % media antes del cambio
mu1 = 1; % media despues del cambio
vari = 1; % varianza

% Hiperparametros del algoritmo Shewart Control Chart
umbral = 0.1; % umbral de deteccion
N=16; % Longitud de los bloques

% Simulacion de Montecarlo
Ns=1e5; % Numero de secuencias analizadas

% Retardos de deteccion del cambio
rd=zeros(Ns,1);

% Bloques en que se detecta el cambio
bd=zeros(Ns,1);

% Instante de cambio (en el primer bloque)
M=zeros(Ns,1);


C1=(mu0^2-mu1^2)/(2*vari);
C2=(mu1-mu0)/vari;


for ns=1:Ns
    
    cambio=0; % inicializa el indicador de cambio
    
    % Primer bloque (bloque en que se produce el cambio)
    M(ns)=randi(N) ; % instante de cambio aleatorio
    x=zeros(N,1);
    x(1:M(ns)-1) = mu0 + sqrt(vari)*randn(M(ns)-1,1);
    x(M(ns):N) = mu1 + sqrt(vari)*randn(N-M(ns)+1,1);
    S=C1+C2*x;
    T=sum(S)/N;
    if T > umbral % deteccion de salto en el primer bloque
        rd(ns) = N-M(ns);
        bd(ns)=1;
        cambio=1;
    end
    % Siguientes bloques
    k=2;
    while cambio ==0
        x = mu1+sqrt(vari)*randn(N,1);
        S=C1+C2*x;
        T=sum(S)/N;        
        if T > umbral
            rd(ns) = k*N-M(ns);
            bd(ns)=k;
            cambio=1;
        end
        k=k+1;
    end
    
    disp(['realizacion: ',num2str(ns),'      M=',num2str(M(ns)),'      rd=',num2str(rd(ns)),'      bloque=',num2str(bd(ns))])
    
end

% Promedio de los retardos de deteccion
ARL1=mean(rd);

disp(' ')
disp(['Longitud bloques: ',num2str(N),'      umbral=',num2str(umbral)])
disp(['ARL1 =',num2str(ARL1)])
